/* splitpathv.idc
 * 
 * Permission to modify and use this code in your own programs is
 * hereby granted provided under the following terms:
 * 1) This notice and all credits, licenses, and mindless blather remain intact.
 * 2) If this is used for professional purposes, the author will be notified as
 *    a courtesy. (ok, I'm curious)
 * 3) Bugfixes and changes to the code or documentaion are freely given back to
 *    the author so they may be shared with everyone.
 * 4) This won't be used to make IDA Pro do evil things.
 *
 * Copyright (c) 2003 Jason Weiler.
 */

#ifndef _SPLITPATHV_IDC_
#define _SPLITPATHV_IDC_

#include <idcrt\lib\strrstr.idc>

#ifndef PATH_DRIVE_IDX
#define PATH_DRIVE_IDX	(0)
#define PATH_DIR_IDX	(1)
#define PATH_FNAME_IDX	(2)
#define PATH_EXT_IDX	(3)
#endif



/* args: path - the input path to be split into its component parts
 *		 outArray - user-allocated array to receive the path-part strings.
 *					The array indices for the various parts are fixed to
 * 					the values defined above.
 * returns: outArray array Id
 * NOTE: the user is responsible for creating and destroying the path array
 */
static splitpathv(path, outArray)
{
	/* outArray[0] - drive
	 * outArray[1] - dir
	 * outArray[2] - fname
	 * outArray[3] - ext
	 * if one of the parts is missing, the index will still exist and be empty
	 */

	auto idx, idx2, tmpStr;

	/* find the drive letter */
	if (substr(path, 1, 2) == ":")
	{
		tmpStr = substr(path, 0, 2);
		
		/* remove the drive letter from the path */
		path = substr(path, 2, -1);
	}
	else
	{
		tmpStr = "";
	}

	SetArrayString(outArray, PATH_DRIVE_IDX, tmpStr);
	
	
	/* now do the directory */
	
	/* find the last slash or backslash */
	idx = strrstr(path, "/");
	idx2 = strrstr(path, "\\");

	if (idx2 > idx)
	{
		idx = idx2;
	}

	if (idx != -1)
	{
		/* there is a path to be had here. Grab it including the last slash */

		idx = idx + 1;  /* we want to keep the trailing slash */
		
		tmpStr = substr(path, 0, idx);
		path = substr(path, idx, -1);
	}
	else
	{
		tmpStr = "";
	}
	
	SetArrayString(outArray, PATH_DIR_IDX, tmpStr);
	
	
	/* time for the filename */
	idx = strrstr(path, ".");
	
	/* conveniently, if there is no period in this string,
		then idx == -1. We'll end up assuming no extension in that case */
	if (idx != 0)
	{
		/* there is an fname to be had here from 0 to idx */
		tmpStr = substr(path, 0, idx);
		
		if (idx != -1)
		{
			path = substr(path, idx, -1);
		}
		else
		{
			path = "";
		}
	}
	else
	{
		tmpStr = "";
	}
	
	SetArrayString(outArray, PATH_FNAME_IDX, tmpStr);

	/* everything else is the extension */
	SetArrayString(outArray, PATH_EXT_IDX, path);

	return outArray;
}

#endif

